//+------------------------------------------------------------------+
//|                                                        Bands.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LightSeaGreen
#property indicator_color2  LightSeaGreen
#property indicator_color3  LightSeaGreen

//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern int    BandsLength     = 20;
extern double BandsDeviation  = 3.0; 
extern int    AppliedPrice    = PRICE_CLOSE;
extern int    BandsMaMode     = MODE_EMA; 

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = true;

//
//
//
//
//

double Ma[];
double UpMa[];
double DnMa[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
  IndicatorBuffers(4);
   SetIndexBuffer(0,Ma);
   SetIndexBuffer(1,UpMa);
   SetIndexBuffer(2,DnMa);
   SetIndexBuffer(3,trend);

      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
  IndicatorShortName(timeFrameToString(timeFrame)+"   bollinger bands ");    
return(0);
}

int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
  {
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-1,Bars-counted_bars-1);
           if (returnBars)  { Ma[0] = limit+1; return(0); }
           
   //
   //
   //
   //
   //
    
   if (calculateValue || timeFrame==Period())
   {       
     for (i = limit; i >= 0; i--)
     {
         double StdDev  = iStdDev(NULL,0,BandsLength,0,BandsMaMode,AppliedPrice,i); 
                Ma[i]   = iMA(NULL,0,BandsLength,0,BandsMaMode,AppliedPrice,i);
                UpMa[i] = Ma[i] + (StdDev*BandsDeviation);
                DnMa[i] = Ma[i] - (StdDev*BandsDeviation);
                trend[i] = trend[i+1]; 
                   if(Close[i]>UpMa[i])  trend[i] = 1;
                   if(Close[i]<DnMa[i])  trend[i] =-1;
       }
       manageAlerts();         
   return(0);            
   }
   
   //
   //
   //
   //
   //
      
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Ma[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",BandsLength,BandsDeviation,AppliedPrice,BandsMaMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
         UpMa[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",BandsLength,BandsDeviation,AppliedPrice,BandsMaMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
         DnMa[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",BandsLength,BandsDeviation,AppliedPrice,BandsMaMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
    }    
return(0);            
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"closed above upper band");
         if (trend[whichBar] == -1) doAlert(whichBar,"closed below lower band");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",timeFrameToString(Period())+" Bollinger Bands price ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Bollinger Bands "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}   

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//


